#include "StdAfx.h"
#include ".\windowscriptletplugin.h"
#include "XMLUtil.h"
#include "ScriptletConstant.h"
#include "WindowAdapterScriptletPlugin.h"
#include "BaseWindow.h"
#include "ScriptletContainerFactory.h"
#include "ScriptletDialog.h"
#include "MessageBox.h"
#include "Application.h"
#include "NButton.h"

CWindowScriptletPlugin::CWindowScriptletPlugin()
{
}

CWindowScriptletPlugin::~CWindowScriptletPlugin(void)
{
}
CRootWindow* CWindowScriptletPlugin::CreateBaseWindow(TiXmlElement* pNode, IDataContext* dataContext)
{
	TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};

	CBaseWindow* pBaseWindow = NULL;
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_CLASS_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_CLASS_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		pBaseWindow = CScriptletContainerFactory::get_instance()->CreateBaseWindow(szValue);
	}
	if (pBaseWindow == NULL)
	{
		pBaseWindow = new CBaseWindow;
	}
	pBaseWindow->SetDataContext(dataContext);
	CWindowScriptletPlugin windowScriptPlugin;
	windowScriptPlugin.CreateBaseWindow(pBaseWindow, pNode);

	return pBaseWindow;
}
INT_PTR CWindowScriptletPlugin::DoModal(TiXmlElement* pNode, HWND hWndParent, LPARAM lParam, IDataContext* dataContext)
{
	CScriptletDialog* pDialog = CretateDialog(pNode);

	pDialog->SetScriptletNode(pNode);
	pDialog->SetDataContext(dataContext);
	INT_PTR ret = pDialog->DoModal(hWndParent, lParam);
	delete pDialog;
	return ret;
}
INT_PTR CWindowScriptletPlugin::DoModalAgreement(TiXmlElement* pNode, HWND hWndParent, LPARAM lParam, IDataContext* dataContext, BOOL& bCoreValueChecked)
{
	CScriptletDialog* pDialog = CretateDialog(pNode);

	pDialog->SetScriptletNode(pNode);
	pDialog->SetDataContext(dataContext);
	INT_PTR ret = pDialog->DoModal(hWndParent, lParam);

	CComponent* corevalueCheck =  pDialog->GetRootContainer()->FindName(_T("set_corevalue_checkbutton"));
	CNButton *pButton = dynamic_cast<CNButton *>(corevalueCheck);
	if (pButton != NULL && pButton->GetPushed())
	{
		bCoreValueChecked = true;
	}
	else
	{
		bCoreValueChecked = false;
	}

	delete pDialog;
	return ret;	
}
INT_PTR CWindowScriptletPlugin::DoModal(CScriptletDialog* pDialog, TiXmlElement* pNode, HWND hWndParent, LPARAM lParam, IDataContext* dataContext)
{
	pDialog->SetScriptletNode(pNode);
	pDialog->SetDataContext(dataContext);
	INT_PTR ret = pDialog->DoModal(hWndParent, lParam);
	delete pDialog;
	return ret;
}
CScriptletDialog* CWindowScriptletPlugin::CretateDialog( TiXmlElement* pNode )
{
	CScriptletDialog* pDialog = NULL;
	TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
	BOOL bCustomDialog = FALSE;
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_CLASS_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_CLASS_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		pDialog = CScriptletContainerFactory::get_instance()->CreateScriptDialog(szValue);
	}
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_CUSTOM_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_CUSTOM_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		bCustomDialog = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
	}
	if ( NULL == pDialog )
	{
		pDialog = new CScriptletDialog(bCustomDialog ? IDD_CUSTOMBASEDIALOG : IDD_BASEDIALOG);
	}
	return pDialog;
}
BOOL CWindowScriptletPlugin::CreateBaseWindow(CRootWindow* pRootWindow, TiXmlElement* pNode)
{
	ATLASSERT(pRootWindow);
	if (pRootWindow == NULL && pNode == NULL) return FALSE;

	DWORD dwStyle = 0;
	DWORD dwStyleEx = 0;

	TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
	long lValue = 0;

	USES_CONVERSION;
	//inherit
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_INHERIT_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		TiXmlElement* pInheritNode = Application::LoadLayoutNode(szValue);
		if ( NULL != pInheritNode )
		{
			CreateBaseWindow(pRootWindow, pInheritNode);
		}
	}
	// wstyle
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_WSTYLE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_WSTYLE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{		
		dwStyle |= CWindowAdapterScriptletPlugin::ParseWindowStyle(szValue);
		dwStyle |= WS_VISIBLE;
	}
	// wexstyle
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_WEXSTYLE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_WEXSTYLE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{		
		dwStyleEx |= CWindowAdapterScriptletPlugin::ParseWindowExStyle(szValue);
	}
	CBaseWindow* pBaseWindow = dynamic_cast<CBaseWindow*>(pRootWindow);
	if (pBaseWindow)
	{
		// [:2008-12-17] Style ߰ Ͽ ModifyStyle ǵ 
		if (pBaseWindow->IsWindow() == FALSE)
			pBaseWindow->Create(NULL, CRect(-100, -100, 1, 1), dwStyle, dwStyleEx);
		else
			pBaseWindow->ModifyStyle(0, dwStyle);
	}	
	
	return ModifyComponent(pRootWindow, pNode);
}
BOOL CWindowScriptletPlugin::CreateBaseDialog(CRootWindow* pRootWindow, TiXmlElement* pNode)
{
	ATLASSERT(pRootWindow);
	if (pRootWindow == NULL && pNode == NULL) return FALSE;

	CWindow window = pRootWindow->GetRootContainer()->GetOwner();
	DWORD dwStyle = 0;
	DWORD dwStyleEx = 0;

	TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
	long lValue = 0;

	//inherit
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_INHERIT_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		TiXmlElement* pInheritNode = Application::LoadLayoutNode(szValue);
		if ( NULL != pInheritNode )
		{
			CreateBaseDialog(pRootWindow, pInheritNode);
		}
	}
	// wstyle
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_WSTYLE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_WSTYLE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{		
		dwStyle |= CWindowAdapterScriptletPlugin::ParseWindowStyle(szValue);
		window.ModifyStyle(0, dwStyle);
	}
	// wexstyle
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_WEXSTYLE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_WEXSTYLE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{		
		dwStyleEx |= CWindowAdapterScriptletPlugin::ParseWindowExStyle(szValue);
		window.ModifyStyleEx(0, dwStyleEx);
	}
	return ModifyComponent(pRootWindow, pNode);
}
BOOL CWindowScriptletPlugin::ModifyComponent(CRootWindow* pRootWindow, TiXmlElement* pNode)
{
	ATLASSERT(pRootWindow);
	if (pRootWindow == NULL && pNode == NULL) return FALSE;

	CWindow windowBase = pRootWindow->GetRootContainer()->GetOwner();

	CRect rcWindow;
	windowBase.GetWindowRect(&rcWindow);
	
	TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
	long lValue = 0;

	USES_CONVERSION;
	// x
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_X_NODE), lValue))
	{
		rcWindow.MoveToX(lValue);
	}
	// y
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_Y_NODE), lValue))
	{
		rcWindow.MoveToY(lValue);
	}
	// width
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_WIDTH_NODE), lValue))
	{
		rcWindow.right = rcWindow.left + lValue;
	}
	// height
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_HEIGHT_NODE), lValue))
	{
		rcWindow.bottom = rcWindow.top + lValue;
	}
	
	// enabled
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_ENABLED_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		BOOL bEnabled = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
		windowBase.EnableWindow(bEnabled);
	}
	// visible
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_VISIBLE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		BOOL bVisible = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
		windowBase.ShowWindow(bVisible ? SW_SHOW : SW_HIDE);
	}
	// caption
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_CAPTION_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_CAPTION_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		LPCTSTR szTitle = LoadString(szValue);
		windowBase.SetWindowText(szTitle);
	}
	// layout
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_LAYOUT_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_LAYOUT_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		ILayoutManager* pLayout = CScriptletContainerFactory::get_instance()->CreateLayout(szValue);
		if ( NULL != pLayout )
		{
			pRootWindow->GetRootContainer()->SetLayoutManager(pLayout);
			// [:2008-12-18] Memory Leak  : SetLayoutManager AutoRelease 
			pLayout->AutoRelease();
		}
	}
	// onclose
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_ONCLOSE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_ONCLOSE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		IWindowListener* pAction = CScriptletContainerFactory::get_instance()->CreateWindowAction(szValue);
		if ( NULL != pAction )
		{
			pRootWindow->SetCloseAction(pAction);
		}
	}
	// onminimize
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_ONMINIMIZE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_ONMINIMIZE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		IActionListener* pAction = CScriptletContainerFactory::get_instance()->CreateAction(szValue);
		if ( NULL != pAction )
		{
			pRootWindow->SetMinimizeAction(pAction);
		}
	}
	// onmaximize
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_ONMAXIMIZE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_ONMAXIMIZE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		IActionListener* pAction = CScriptletContainerFactory::get_instance()->CreateAction(szValue);
		if ( NULL != pAction )
		{
			pRootWindow->SetMaximizeAction(pAction);
		}
	}
	// region.round
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_REGION_ROUND_NODE), lValue) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_REGION_ROUND_NODE), lValue))
	{
		pRootWindow->SetRoundRegion(lValue);
	}
	ParseChild(pRootWindow, pNode);
	windowBase.MoveWindow(&rcWindow);	
	// centerwindow
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_CENTERWINDOW_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_CENTERWINDOW_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		BOOL bCenterWindow = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
		pRootWindow->SetCenterWindow(bCenterWindow);
	}

	

	return TRUE;
}
BOOL CWindowScriptletPlugin::ParseChild(CRootWindow* pRootWindow, TiXmlElement* pNode)
{
	//refactoring
	CContainer* pRootContainer = pRootWindow->GetRootContainer();

	USES_CONVERSION;
	TiXmlElement* pChildNode = pNode->FirstChildElement();
	while (pChildNode)
	{
		CScriptletContainerFactory::get_instance()->CreateComponent(pRootContainer, pChildNode);
		
		pChildNode = pChildNode->NextSiblingElement();
	}
	return TRUE;
}